@@ -4,14 +4,14 @@ from math import isnan, isinf
 import operator
 from collections import namedtuple, Sequence, defaultdict
 from contextlib import contextmanager
-from functools import reduce, partial, lru_cache
+from functools import reduce, partial, lru_cache, wraps
 from itertools import chain
 from warnings import warn
 from xml.sax.saxutils import escape
 
 from AnyQt.QtCore import (
-    Qt, QAbstractListModel, QAbstractTableModel, QModelIndex,
-    QItemSelectionModel, QByteArray
+    Qt, QObject, QAbstractListModel, QAbstractTableModel, QModelIndex,
+    QItemSelectionModel
 )
 from AnyQt.QtCore import pyqtSignal as Signal
 from AnyQt.QtGui import QColor
@@ -118,7 +118,7 @@ class AbstractSortTableModel(QAbstractTableModel):
 
         Parameters
         ----------
-        rows : int or list of int or numpy.ndarray or Ellipsis
+        rows : int or list of int or numpy.ndarray of dtype=int or Ellipsis
             View (sorted) rows.
 
         Returns
@@ -127,7 +127,10 @@ class AbstractSortTableModel(QAbstractTableModel):
             Source rows matching input rows. If they are the same,
             simply input `rows` is returned.
         """
-        if self.__sortInd is not None:
+        # self.__sortInd[rows] fails if `rows` is an empty list or array
+        if self.__sortInd is not None \
+                and (isinstance(rows, (Integral, type(Ellipsis)))
+                     or len(rows)):
             new_rows = self.__sortInd[rows]
             if rows is Ellipsis:
                 new_rows.setflags(write=False)
@@ -139,7 +142,7 @@ class AbstractSortTableModel(QAbstractTableModel):
 
         Parameters
         ----------
-        rows : int or list of int or numpy.ndarray or Ellipsis
+        rows : int or list of int or numpy.ndarray of dtype=int or Ellipsis
             Source model rows.
 
         Returns
@@ -148,7 +151,10 @@ class AbstractSortTableModel(QAbstractTableModel):
             ModelIndex (sorted) rows matching input source rows.
             If they are the same, simply input `rows` is returned.
         """
-        if self.__sortIndInv is not None:
+        # self.__sortInd[rows] fails if `rows` is an empty list or array
+        if self.__sortIndInv is not None \
+                and (isinstance(rows, (Integral, type(Ellipsis)))
+                     or len(rows)):
             new_rows = self.__sortIndInv[rows]
             if rows is Ellipsis:
                 new_rows.setflags(write=False)
@@ -177,7 +183,7 @@ class AbstractSortTableModel(QAbstractTableModel):
             indices = indices[::-1]
         return indices
 
-    def sort(self, column: int, order: Qt.SortOrder=Qt.AscendingOrder):
+    def sort(self, column: int, order: Qt.SortOrder = Qt.AscendingOrder):
         """
         Sort the data by `column` into `order`.
 
@@ -368,8 +374,8 @@ class PyTableModel(AbstractSortTableModel):
     def removeRows(self, row, count, parent=QModelIndex()):
         if not parent.isValid():
             del self[row:row + count]
-            for row in range(row, row + count):
-                self._roleData.pop(row, None)
+            for rowidx in range(row, row + count):
+                self._roleData.pop(rowidx, None)
             return True
         return False
 
@@ -500,12 +506,16 @@ class PyListModel(QAbstractListModel):
         if iterable is not None:
             self.extend(iterable)
 
-    def _is_index_valid_for(self, index, list_like):
+    def _is_index_valid(self, index):
+        # This error would happen if one wraps a list into a model and then
+        # modifies a list instead of a model
+        if len(self) != len(self._other_data):
+            raise RuntimeError("Mismatched length of model and its _other_data")
         if isinstance(index, QModelIndex) and index.isValid():
             row, column = index.row(), index.column()
-            return 0 <= row < len(list_like) and not column
+            return 0 <= row < len(self) and column == 0
         elif isinstance(index, int):
-            return -len(self) < index < len(list_like)
+            return -len(self) <= index < len(self)
         else:
             return False
 
@@ -521,7 +531,7 @@ class PyListModel(QAbstractListModel):
 
     # noinspection PyMethodOverriding
     def index(self, row, column=0, parent=QModelIndex()):
-        if self._is_index_valid_for(row, self) and column == 0:
+        if self._is_index_valid(row) and column == 0:
             return QAbstractListModel.createIndex(self, row, column, parent)
         else:
             return QModelIndex()
@@ -541,14 +551,14 @@ class PyListModel(QAbstractListModel):
     def data(self, index, role=Qt.DisplayRole):
         row = index.row()
         if role in [self.list_item_role, Qt.EditRole] \
-                and self._is_index_valid_for(index, self):
+                and self._is_index_valid(index):
             return self[row]
-        elif self._is_index_valid_for(row, self._other_data):
+        elif self._is_index_valid(row):
             return self._other_data[row].get(role, None)
 
     def itemData(self, index):
         mapping = QAbstractListModel.itemData(self, index)
-        if self._is_index_valid_for(index, self._other_data):
+        if self._is_index_valid(index):
             items = list(self._other_data[index.row()].items())
         else:
             items = []
@@ -560,31 +570,31 @@ class PyListModel(QAbstractListModel):
         return QModelIndex()
 
     def setData(self, index, value, role=Qt.EditRole):
-        if role == Qt.EditRole and self._is_index_valid_for(index, self):
-            self[index.row()] = value  # Will emit proper dataChanged signal
-            return True
-        elif self._is_index_valid_for(index, self._other_data):
+        if role == Qt.EditRole:
+            if self._is_index_valid(index):
+                self[index.row()] = value  # Will emit proper dataChanged signal
+                return True
+        elif self._is_index_valid(index):
             self._other_data[index.row()][role] = value
             self.dataChanged.emit(index, index)
             return True
-        else:
-            return False
+        return False
 
     def setItemData(self, index, data):
         data = dict(data)
         with signal_blocking(self):
             for role, value in data.items():
                 if role == Qt.EditRole and \
-                        self._is_index_valid_for(index, self):
+                        self._is_index_valid(index):
                     self[index.row()] = value
-                elif self._is_index_valid_for(index, self._other_data):
+                elif self._is_index_valid(index):
                     self._other_data[index.row()][role] = value
 
         self.dataChanged.emit(index, index)
         return True
 
     def flags(self, index):
-        if self._is_index_valid_for(index, self._other_data):
+        if self._is_index_valid(index):
             return self._other_data[index.row()].get("flags", self._flags)
         else:
             return self._flags | Qt.ItemIsDropEnabled
@@ -658,21 +668,23 @@ class PyListModel(QAbstractListModel):
 
     def __add__(self, iterable):
         new_list = PyListModel(list(self._list),
-                               self.parent(),
+                               # method parent is overloaded in Model
+                               QObject.parent(self),
                                flags=self._flags,
                                list_item_role=self.list_item_role,
-                               supportedDropActions=self.supportedDropActions()
-                               )
+                               supportedDropActions=self.supportedDropActions())
+        # pylint: disable=protected-access
         new_list._other_data = list(self._other_data)
         new_list.extend(iterable)
         return new_list
 
     def __iadd__(self, iterable):
         self.extend(iterable)
+        return self
 
     def __delitem__(self, s):
         if isinstance(s, slice):
-            start, stop, step = _as_contiguous_range(s, len(self))
+            start, stop, _ = _as_contiguous_range(s, len(self))
             self.beginRemoveRows(QModelIndex(), start, stop - 1)
         else:
             s = operator.index(s)
@@ -796,7 +808,7 @@ class VariableListModel(PyListModel):
         self.placeholder = placeholder
 
     def data(self, index, role=Qt.DisplayRole):
-        if self._is_index_valid_for(index, self):
+        if self._is_index_valid(index):
             var = self[index.row()]
             if var is None and role == Qt.DisplayRole:
                 return self.placeholder or "None"
@@ -855,11 +867,6 @@ class VariableListModel(PyListModel):
         text += self.variable_labels_tooltip(var)
         return text
 
-    def python_variable_tooltip(self, var):
-        text = "<b>%s</b><br/>Python" % safe_text(var.name)
-        text += self.variable_labels_tooltip(var)
-        return text
-
 
 class DomainModel(VariableListModel):
     ATTRIBUTES, CLASSES, METAS = 1, 2, 4
@@ -869,8 +876,25 @@ class DomainModel(VariableListModel):
                  ATTRIBUTES)
     PRIMITIVE = (DiscreteVariable, ContinuousVariable)
 
-    def __init__(self, order=SEPARATED, placeholder=None,
+    def __init__(self, order=SEPARATED, separators=True, placeholder=None,
                  valid_types=None, alphabetical=False, skip_hidden_vars=True, **kwargs):
+        """
+
+        Parameters
+        ----------
+        order: tuple or int
+            Order of attributes, metas, classes, separators and other options
+        separators: bool
+            If False, remove separators from `order`.
+        placeholder: str
+            The text that is shown when no variable is selected
+        valid_types: tuple
+            (Sub)types of `Variable` that are included in the model
+        alphabetical: bool
+            If true, variables are sorted alphabetically.
+        skip_hidden_vars: bool
+            If true, variables marked as "hidden" are skipped.
+        """
         super().__init__(placeholder=placeholder, **kwargs)
         if isinstance(order, int):
             order = (order,)
@@ -880,10 +904,13 @@ class DomainModel(VariableListModel):
             order = (None,) + \
                     (self.Separator, ) * (self.Separator in order) + \
                     order
+        if not separators:
+            order = [e for e in order if e is not self.Separator]
         self.order = order
         self.valid_types = valid_types
         self.alphabetical = alphabetical
         self.skip_hidden_vars = skip_hidden_vars
+        self._within_set_domain = False
         self.set_domain(None)
 
     def set_domain(self, domain):
@@ -920,9 +947,65 @@ class DomainModel(VariableListModel):
                     content.append(self.Separator)
                     add_separator = False
                 content += to_add
-        self[:] = content
+        try:
+            self._within_set_domain = True
+            self[:] = content
+        finally:
+            self._within_set_domain = False
         self.endResetModel()
 
+    def prevent_modification(method):  # pylint: disable=no-self-argument
+        @wraps(method)
+        # pylint: disable=protected-access
+        def e(self, *args, **kwargs):
+            if self._within_set_domain:
+                method(self, *args, **kwargs)
+            else:
+                raise TypeError(
+                    "{} can be modified only by calling 'set_domain'".
+                    format(type(self).__name__))
+        return e
+
+    @prevent_modification
+    def extend(self, iterable):
+        return super().extend(iterable)
+
+    @prevent_modification
+    def append(self, item):
+        return super().append(item)
+
+    @prevent_modification
+    def insert(self, i, val):
+        return super().insert(i, val)
+
+    @prevent_modification
+    def remove(self, val):
+        return super().remove(val)
+
+    @prevent_modification
+    def pop(self, i):
+        return super().pop(i)
+
+    @prevent_modification
+    def clear(self):
+        return super().clear()
+
+    @prevent_modification
+    def __delitem__(self, s):
+        return super().__delitem__(s)
+
+    @prevent_modification
+    def __setitem__(self, s, value):
+        return super().__setitem__(s, value)
+
+    @prevent_modification
+    def reverse(self):
+        return super().reverse()
+
+    @prevent_modification
+    def sort(self, *args, **kwargs):
+        return super().sort(*args, **kwargs)
+
 
 _html_replace = [("<", "&lt;"), (">", "&gt;")]
 
@@ -1051,6 +1134,8 @@ class TableModel(AbstractSortTableModel):
     Basket = namedtuple(
         "Basket", ["vars", "role", "background", "density", "format"])
 
+    # The class uses the same names (X_density etc) as Table
+    # pylint: disable=invalid-name
     def __init__(self, sourcedata, parent=None):
         super().__init__(parent)
         self.source = sourcedata
@@ -1080,7 +1165,7 @@ class TableModel(AbstractSortTableModel):
             elif role == TableModel.ClassVar:
                 getter = operator.attrgetter("sparse_y")
             elif role == TableModel.Meta:
-                getter = operator.attrgetter("sparse_meta")
+                getter = operator.attrgetter("sparse_metas")
             return partial(formater, vars, getter)
 
         def make_basket(vars, density, role):
@@ -1188,15 +1273,14 @@ class TableModel(AbstractSortTableModel):
              _VariableStatsRole=VariableStatsRole,
              # Some cached local precomputed values.
              # All of the above roles we respond to
-             _recognizedRoles=set([Qt.DisplayRole,
-                                   Qt.EditRole,
-                                   Qt.BackgroundRole,
-                                   ValueRole,
-                                   ClassValueRole,
-                                   VariableRole,
-                                   DomainRole,
-                                   VariableStatsRole]),
-             ):
+             _recognizedRoles=frozenset([Qt.DisplayRole,
+                                         Qt.EditRole,
+                                         Qt.BackgroundRole,
+                                         ValueRole,
+                                         ClassValueRole,
+                                         VariableRole,
+                                         DomainRole,
+                                         VariableStatsRole])):
         """
         Reimplemented from `QAbstractItemModel.data`
         """
@@ -1226,7 +1310,6 @@ class TableModel(AbstractSortTableModel):
             return instance[coldesc.var]
         elif role == _Qt_BackgroundRole:
             return coldesc.background
-            return self.color_for_role[coldesc.role]
         elif role == _ValueRole and isinstance(coldesc, TableModel.Column):
             return instance[coldesc.var]
         elif role == _ClassValueRole:
